#!/usr/bin/python
# encoding: utf-8
#
"""
Copyright or © or Copr. Baptiste Chupin (2013-07-15)

baptiste.chupin@obspm.fr

This software is a computer program whose purpose is to launch with a user 
interface the soft of common-views.

This software is governed by the [CeCILL|CeCILL-B|CeCILL-C] license under 
French law and abiding by the rules of distribution of free software. 
You can use, modify and/ or redistribute the software under the terms of the 
[CeCILL|CeCILL-B|CeCILL-C] license as circulated by CEA, CNRS and INRIA at the 
following URL "http://www.cecill.info". 

As a counterpart to the access to the source code and rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author, the holder of the
economic rights, and the successive licensors have only limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading, using, modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean that it is complicated to manipulate, and that also
therefore means that it is reserved for developers and experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and, more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the [CeCILL|CeCILL-B|CeCILL-C] license and that you accept its 
terms.

Author: Baptiste Chupin
Version: 1.0
Date: 07/19/2013

$Id: lcComView.py 493 2017-09-13 15:25:36Z chupin $

"""
#
import os
import sys
import tempfile
import platform
import xml.dom.minidom
import datetime
# Import modules locaux
import timeconv
# Import des modules non inclus dans distrib python
try :
    from lxml import etree
except ImportError :
    print("""
    You need to install python-lxml module
    In a terminal, type :
        sudo apt-get install python-lxml
    """)
    sys.exit(-3)
try :
    import wx
except ImportError :
    print("""
    You need to install python-wxgtk module
    In a terminal, type :
        sudo apt-get install python-wxgtk
    """)
    sys.exit(-3)
# Local
import main as cpr
#
ID_PATH=''
SCH_PATH='schemaconfig.xsd'
WX_SIZE=(960,740)
# Definition Date
YEST=timeconv.mjd(datetime.datetime.now()-
                  datetime.timedelta(days=1),"string")
DEFAULT_DEB=YEST
DEFAULT_FIN=YEST
#
if platform.system()=='Linux' :
    SLASH='/'
elif platform.system()=='Windows' :
    SLASH='\\'
#
compglo={}
configglo={}
lopngglo=[]
#
def formatXML(xmlFile) :
    f=tempfile.NamedTemporaryFile(suffix=".tmp").name
    cmd="xmllint --format %s > %s"%(xmlFile,f)
    os.popen(cmd)
    cmd="mv %s %s"%(f,xmlFile)
    os.popen(cmd)
    return -1
#
def scaleBitmap(bitmap,width,height) :
    image=wx.ImageFromBitmap(bitmap)
    image=image.Scale(width,height,wx.IMAGE_QUALITY_HIGH)
    result=wx.BitmapFromImage(image)
    return result
#
class MainFrame(wx.Frame) :
    def __init__(self,parent,id,title) :
        self.dom=None
        self.pathconfig=None
        self.config={}
        self.comp=[]
        self.options=""
        self.laboremoved=None
        self.loboselected=None
        self.lofgs=None
        self.lopng=None
        self.schemaXML_doc=etree.parse(SCH_PATH)
        self.schemaXML=etree.XMLSchema(self.schemaXML_doc)
        wx.Frame.__init__(self,parent,id,title,size=WX_SIZE)
#
        # Status Bar
        self.sb=self.CreateStatusBar()
        # Logos
        lgCnrs=wx.Bitmap('cnrs.png')
        lgCnrs=scaleBitmap(lgCnrs,60,60)
        ctrlLgCnrs=wx.StaticBitmap(self,-1,lgCnrs)
        ctrlLgCnrs.SetPosition((WX_SIZE[0]-80,10))
        lgLne=wx.Bitmap('lne.png')
        lgLne=scaleBitmap(lgLne,130,60)
        ctrlLgLne=wx.StaticBitmap(self,-1,lgLne)
        ctrlLgLne.SetPosition((WX_SIZE[0]-230,10))
        lgObs=wx.Bitmap('syrte.png')
        lgObs=scaleBitmap(lgObs,170,60)
        ctrlLgObs=wx.StaticBitmap(self,-1,lgObs)
        ctrlLgObs.SetPosition((WX_SIZE[0]-420,10))
        # Barre Menu
        menuBar=wx.MenuBar()
        fich=wx.Menu()
        fich.Append(101,'&New\tCtrl+N','Create a new config file')
        fich.Append(102,'&Open\tCtrl+O','Open a config file')
        fich.Append(103,'&Save\tCtrl+S','Save the config file')
        fich.Append(104,'&Save as...\tShift+Ctrl+S','Save the config file')
        fich.AppendSeparator()
        quit=wx.MenuItem(fich,105,'&Quit\tCtrl+Q','Quit the Application')
        fich.AppendItem(quit)
        menuBar.Append(fich,'&File')
        edit=wx.Menu()
        edit.Append(201,'&Add Comp\tCtrl+A',
                    'Add a recepteur compared to the ref')
        edit.Append(202,'&Remove Comp\tCtrl+R',
                    'Remove a recepteur compared to the ref')
        menuBar.Append(edit,'&Edit')
        #
        configure=wx.Menu()
        configure.Append(301,'&Check XML\tCtrl+C',
                         'Check the XML FILE')
        menuBar.Append(configure,'&XML')
        #
        plotM=wx.Menu()
        plotM.Append(401,'&Generate plot scripts\tCtrl+G',
                    'Genrate the gnuplot scripts for comparisons')
        plotM.Append(402,'&Generate plots\tCtrl+P',
                     'Generate the plot of the comparisons')
        plotM.AppendSeparator()
        plotM.Append(403,'&View plot\tCtrl+V',
                     'View the plots of comparisons')
        menuBar.Append(plotM,'&Plot')
        self.SetMenuBar(menuBar)
        # Event du Menu Bar
        # File 
        self.Bind(wx.EVT_MENU,self.OnQuit,id=105)
        self.Bind(wx.EVT_MENU,self.OnNewFile,id=101)
        self.Bind(wx.EVT_MENU,self.OnOpenFile,id=102)
        self.Bind(wx.EVT_MENU,self.OnSaveFile,id=103)
        self.Bind(wx.EVT_MENU,self.OnSaveNewFile,id=104)
        # Edit
        self.Bind(wx.EVT_MENU,self.OnAddComp,id=201)
        self.Bind(wx.EVT_MENU,self.OnRemoveComp,id=202)
        # Configuration
        self.Bind(wx.EVT_MENU,self.OnCheckXML,id=301)
        # Plot 
        self.Bind(wx.EVT_MENU,self.OnMkScptPlt,id=401)
        self.Bind(wx.EVT_MENU,self.OnMkPlt,id=402)
        self.Bind(wx.EVT_MENU,self.OnViewPng,id=403)
#
        # Boites config
        wx.StaticText(self,-1,'Config file pathname',(20,10))
        wx.StaticLine(self,-1,(180,10),(320,3))
        self.configSet=wx.TextCtrl(self,-1,'',(20,30),(485,-1))
#        wx.StaticText(self,-1,'Modifications saved',(20,65))
        self.savedSet=wx.TextCtrl(self,-1,'',(20,60),(125,-1))
#        self.Bind(wx.EVT_KEY_UP,self.OnEventKey)
        self.Bind(wx.EVT_COMBOBOX,self.OnEventAction)
        self.Bind(wx.EVT_TEXT_ENTER,self.OnEventAction)
        wx.StaticLine(self,-1,(20,90),(480,3))
#
        # Parametres reference
        # Texte des «boites»
        wx.StaticText(self,-1,'Ref directory',(20,110))
        wx.StaticText(self,-1,'Ref receiver',(20,150))
        wx.StaticText(self,-1,'Format',(205,150))
        wx.StaticText(self,-1,'Sigma',(365,150))
        wx.StaticText(self,-1,'Ordre fit',(465,150))
        wx.StaticText(self,-1,'Elv filt',(585,150))
        wx.StaticText(self,-1,'Comparisons',(20,190))
        wx.StaticText(self,-1,'Process',(460,430))
        wx.StaticText(self,-1,'Begin',(25,430))
        wx.StaticText(self,-1,'End',(115,430))
        wx.StaticText(self,-1,'iterations',(160,490))
        # Def des «boites»
        self.refdir=wx.TextCtrl(self,-1,'',(125,105),(440,-1),
                                style=wx.TE_PROCESS_ENTER)
        self.refrec=wx.TextCtrl(self,-1,'gz****',(125,145),(70,-1),
                                style=wx.TE_PROCESS_ENTER)
        self.cgtts =wx.ComboBox(self,-1,'CGGTTS4',(260,145),(100,-1),
                                choices=['CGGTTS4','gps','gal','glo','mix'])
        self.sigma =wx.ComboBox(self,-1,'3',(410,145),(50,-1),
                                choices=['1','2','3','4','5','6'])
        self.ordre =wx.ComboBox(self,-1,'1',(530,145),(50,-1),
                                choices=['1','2','3'])
        self.elvfil=wx.TextCtrl(self,-1,'0',(630,145),(40,-1),
                                style=wx.TE_PROCESS_ENTER)
        # Bouton Browse Ref
        broRef=wx.Button(self,1,'Browse',(580,100))
        self.Bind(wx.EVT_BUTTON,self.SearchDirRef,id=1)
#
        # Boite de check du XML
        self.check=wx.TextCtrl(self,-1,'',(690,100),size=(250,75),
                               style=wx.TE_MULTILINE | wx.TE_PROCESS_ENTER 
                               | wx.TE_READONLY)
#
        wx.StaticLine(self,-1,(20,185),(920,3))
#
        # Boite pour la liste des comps
        self.lc=wx.ListCtrl(self,-1,(20,210),size=(920,200),style=wx.LC_REPORT)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED,self.OnModifieCompByDC,self.lc)
        self.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK,self.ContextMenu)
        self.lc.InsertColumn(0,'Name')
        self.lc.SetColumnWidth(0,60)
        self.lc.InsertColumn(1,'Raw directory')
        self.lc.SetColumnWidth(1,290)
        self.lc.InsertColumn(2,'Receiver')
        self.lc.SetColumnWidth(2,60)
        self.lc.InsertColumn(3,'Out directory')
        self.lc.SetColumnWidth(3,290)
        self.lc.InsertColumn(4,'Out file')
        self.lc.SetColumnWidth(4,105)
        self.lc.InsertColumn(5,'Elv filt')
        self.lc.SetColumnWidth(5,50)
        self.lc.InsertColumn(6,'Format')
        self.lc.SetColumnWidth(6,65)
#
        wx.StaticLine(self,-1,(20,420),(920,3))
#
        # Lancement Comparaison
        # Boite recuperation sortie standart
        # Parametres date
        self.bdeb=wx.SpinCtrl(self,-1,DEFAULT_DEB,(20,450),(75,-1),
                              min=50000,max=99999)
        self.bfin=wx.SpinCtrl(self,-1,DEFAULT_FIN,(110,450),(75,-1),
                              min=50000,max=99999)
        # Parametres arguments
        self.bxstop=wx.CheckBox(self,-1,'Stop at',(20,485))
        self.bcnstop=wx.SpinCtrl(self,-1,"1",(110,485),(45,-1),
                                 min=1,max=99)
        self.bxwsat=wx.CheckBox(self,-1,'Save satellites files',(20,515))
        self.bxmad=wx.CheckBox(self,-1,'Methode MAD-Based',(20,545))
        self.bxepoch=wx.CheckBox(self,-1,'Remove epochs outliers',(20,575))
        self.bxelog=wx.CheckBox(self,-1,'Save a log file',(20,605))
        # Bouton Run 
        run=wx.Button(self,9,'Run',(20,630))
        self.Bind(wx.EVT_BUTTON,self.RunComp,id=9)
        # jauge
        self.gauge=wx.Gauge(self,-1,pos=(130,640),size=(315,25))
        # Rediretion stdout vers la «boite» self.log 
        self.log=wx.TextCtrl(self,-1,'',(460,450),size=(480,215),
                             style=wx.TE_MULTILINE | wx.TE_PROCESS_ENTER | 
                             wx.TE_READONLY)
        self.log.SetForegroundColour((255,255,230))
        self.log.SetBackgroundColour((050,050,050))
        font1=wx.Font(10,wx.MODERN,wx.NORMAL,wx.NORMAL,False,u'Monospace')
        self.log.SetFont(font1)
        redir=RedirectText(self.log)
        sys.stdout=redir
#
#        self.Centre()
#
    def ContextMenu(self,event) :
        self.loboselected=event.GetText()
        contextuel=wx.Menu()
        contextuel.Append(991,'Remove')
        contextuel.Append(992,'Modifie')
        contextuel.AppendSeparator()
        contextuel.Append(993,'New')
        self.Bind(wx.EVT_MENU,self.OnRemoveCompRC,id=991)
        self.Bind(wx.EVT_MENU,self.OnModifieCompByDC,id=992)
        self.Bind(wx.EVT_MENU,self.OnAddComp,id=993)
        position=(event.GetPoint()[0]+20,event.GetPoint()[1]+170)
        self.PopupMenu(contextuel,pos=position)
        contextuel.Destroy()
#
    def OnEventKey(self,event) :
        self.savedSet.SetForegroundColour((255,255,230))
        self.savedSet.SetBackgroundColour((255,000,000))
        self.savedSet.SetValue("Config modified")
#   
    def OnEventAction(self,event) :
        self.savedSet.SetForegroundColour((255,255,230))
        self.savedSet.SetBackgroundColour((255,000,000))
        self.savedSet.SetValue("Config modified")
#
    def OnMkScptPlt(self,event) :
        if (self.comp==[]) :
            dlg=wx.MessageDialog(self,"Create, save or load a XML file please")
            dlg.ShowModal()
            dlg.Destroy()
            return False
        else :
            p=ID_PATH
            dlg=wx.DirDialog(self, "Choose a directory",
                             defaultPath=p, 
                             style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
            if dlg.ShowModal() == wx.ID_OK:
                gnudir=dlg.GetPath()
            else :
                dlg.Destroy()
                return False
            lofdne=[]
            lofgs=[]
            lopng=[]
            for compI in self.comp :
                filename=compI["dir_out"]+compI["file_out"]
                if(os.path.isfile(filename)) :
                    gnufile=gnudir+SLASH+compI["file_out"][:-3]+"plt"
                    f=open(gnufile,'w')
                    f.write("#!/usr/bin/gnuplot\n")
                    f.write("set terminal png nocrop enhanced fontscale 1.5 size 960,540\n")
                    pngfile="%spng"%gnufile[:-3]
                    f.write("set o '%s'\n"%pngfile)
                    lopng.append(pngfile)
                    f.write("set title '%s-%s'\n"
                            %(compI["file_comp"],self.config["file_ref"]))
                    f.write("set xlabel 'MJD / d'\n")
                    f.write("set ylabel 'Phase differences / ns'\n")
                    f.write("set g\n")
                    f.write("unset key\n")
                    f.write("plot '%s' w lp\n"%filename)
                    f.write("quit")
                    f.close()
                    lofgs.append(gnufile)
                else :
                    lofdne.append(filename)
            if(lofdne!=[]) :
                message="Le(s) fichier(s) "
                for f in lofdne :
                    message+="%s "%f
                message+="n'existe(nt) pas"
                dlg=wx.MessageDialog(self,message)
                dlg.ShowModal()
                dlg.Destroy()
                return False
        self.lofgs=lofgs
        self.lopng=lopng
        return True
#
    def OnMkPlt(self,event) :
        try :
            for f in self.lofgs :
                os.system('gnuplot %s'%f)
            return True
        except TypeError :
            message="Create the gnuplot scripts please"
            dlg=wx.MessageDialog(self,message)
            dlg.ShowModal()
            dlg.Destroy()
            return False
#
    def OnViewPng(self,event) :
        self.sb.SetStatusText("View png plot of comparisons")
        addframe=ViewPng(None,-1,"View plot",self.lopng)
        addframe.Show(True)
#
    def OnCheckXML(self,event) :
        try :
            xml=etree.parse(self.pathconfig)
            self.valid=self.schemaXML.validate(xml)
            if not self.valid :
                log=self.schemaXML.error_log
                self.check.SetForegroundColour((255,255,230))
                self.check.SetBackgroundColour((255,000,000))
                NoE=len(log)
                st="Number of errors : %i in %s"%(NoE,self.pathconfig)
                for l in log :
                    st+="\n"+str(l.type_name)
                self.check.SetValue(st)
                return False
            else :
                self.check.SetForegroundColour((255,255,230))
                self.check.SetBackgroundColour((050,000,205))
                st="%s is OK"%(self.pathconfig)
                self.check.SetValue(st)
                return True
        except TypeError :
            dlg=wx.MessageDialog(self,"Create or load a XML file please")
            dlg.ShowModal()
            dlg.Destroy()
            return False
#
    def SearchDirRef(self,event) :
        p=ID_PATH
        dlg = wx.DirDialog(self, "Choose a directory",
                           defaultPath=p, 
                           style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if dlg.ShowModal() == wx.ID_OK:
            self.RefDirV=dlg.GetPath()
            self.refdir.SetValue(self.RefDirV+SLASH)
        dlg.Destroy()
#
    def OnNewFile(self,event) :
        self.dom=None
        self.pathconfig=None
        self.config={}
        self.comp=[]
        self.options=""
        self.laboremoved=None
        self.loboselected=None
        self.lofgs=None
        self.lopng=None
        self.SetRef()
        self.SetComp()
#
    def OnOpenFile(self,event) :
        wcd='XML files (*.xml)|*.xml|All files (*)|*'
        p=ID_PATH
        open_dlg=wx.FileDialog(self, message='Choose a file',
                               defaultDir=p,defaultFile='',
                               wildcard=wcd, style=wx.OPEN|wx.CHANGE_DIR)
        openVar=open_dlg.ShowModal()
        if openVar==wx.ID_OK :
            path=open_dlg.GetPath()
            self.pathconfig=path
            try :
                self.dom=xml.dom.minidom.parse(path)
            except IOError,error :
                dlg=wx.MessageDialog(self,'Error opening file\n'+str(error))
                dlg.ShowModal()
                dlg.Destroy()
                open_dlg.Destroy()
                return False
            except xml.parsers.expat.ExpatError :
                dlg=wx.MessageDialog(self,'Le fichier %s est vide'%(path))
                dlg.ShowModal()
                dlg.Destroy()
                open_dlg.Destroy()
                return False
            self.config={}
            self.comp=[]
            self.ParseDom()
            open_dlg.Destroy()
        elif openVar==wx.ID_CANCEL :
            open_dlg.Destroy()
            return False
        self.configSet.SetValue(self.pathconfig)
        self.savedSet.SetForegroundColour((255,255,230))
        self.savedSet.SetBackgroundColour((050,000,205))
        self.savedSet.SetValue("Config saved")
        self.check.SetValue("")
        self.check.SetForegroundColour((000,000,000))
        self.check.SetBackgroundColour((255,255,255))
        self.lofgs=None
        self.lopng=None
        return True
        open_dlg.Destroy()
#
    def OnSaveNewFile(self,event) :
        self.OnValidRef(event)
        if (self.config!={}) and (self.comp!=[]) :
            xmlOut=xml.dom.minidom.Document()
            racine=xmlOut.createElement("config")
            xmlOut.appendChild(racine)
            param=xmlOut.createElement("params")
            racine.appendChild(param)
            for c in self.config :
                element=xmlOut.createElement(c)
                param.appendChild(element)
                text=xmlOut.createTextNode(self.config[c])
                element.appendChild(text)
            for c in self.comp :
                comparaison=xmlOut.createElement("comparaison")
                racine.appendChild(comparaison)
                for sc in c :
                    element=xmlOut.createElement(sc)
                    comparaison.appendChild(element)
                    text=xmlOut.createTextNode(c[sc])
                    element.appendChild(text)
            wcd='XML files (*.xml)|*.xml|All files (*)|*'
            save_dlg=wx.FileDialog(self, message='Save file as',
                                defaultDir='',defaultFile='',
                                wildcard=wcd, style=wx.SAVE)
            if save_dlg.ShowModal()==wx.ID_OK :
                path=save_dlg.GetPath()
                f=open(path,'w')
                f.write(xmlOut.toxml(encoding='utf-8'))
                f.close()
                self.pathconfig=path
                self.configSet.SetValue(self.pathconfig)
                self.savedSet.SetForegroundColour((255,255,230))
                self.savedSet.SetBackgroundColour((050,000,205))
                self.savedSet.SetValue("Config saved")
                formatXML(path)
        elif self.config=={} :
            dlg=wx.MessageDialog(self,
                                 'There are no reference created or loaded',
                                 'Message', wx.OK|wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return True
        elif self.comp==[] :
            dlg=wx.MessageDialog(self,
                                 'There are no comparison created or loaded',
                                 'Message', wx.OK|wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return True
        return True
#
    def OnSaveFile(self,event) :
        self.OnValidRef(event)
        if (self.config!={}) and (self.comp!=[]) :
            xmlOut=xml.dom.minidom.Document()
            racine=xmlOut.createElement("config")
            xmlOut.appendChild(racine)
            param=xmlOut.createElement("params")
            racine.appendChild(param)
            for c in self.config :
                element=xmlOut.createElement(c)
                param.appendChild(element)
                text=xmlOut.createTextNode(self.config[c])
                element.appendChild(text)
            for c in self.comp :
                comparaison=xmlOut.createElement("comparaison")
                racine.appendChild(comparaison)
                for sc in c :
                    element=xmlOut.createElement(sc)
                    comparaison.appendChild(element)
                    text=xmlOut.createTextNode(c[sc])
                    element.appendChild(text)
            if (not self.pathconfig) :
                wcd='XML files (*.xml)|*.xml|All files (*)|*'
                save_dlg=wx.FileDialog(self, message='Save file as',
                                    defaultDir='',defaultFile='',
                                    wildcard=wcd, style=wx.SAVE)
                if save_dlg.ShowModal()==wx.ID_OK :
                    path=save_dlg.GetPath()
                    f=open(path,'w')
                    f.write(xmlOut.toxml(encoding='utf-8'))
                    f.close()
                    self.pathconfig=path
                    self.configSet.SetValue(self.pathconfig)
                    self.savedSet.SetForegroundColour((255,255,230))
                    self.savedSet.SetBackgroundColour((050,000,205))
                    self.savedSet.SetValue("Config saved")
                    formatXML(path)
            else :
                path=self.pathconfig
                f=open(path,'w')
                f.write(xmlOut.toxml(encoding='utf-8'))
                f.close()
                self.pathconfig=path
                self.configSet.SetValue(self.pathconfig)
                self.savedSet.SetForegroundColour((255,255,230))
                self.savedSet.SetBackgroundColour((050,000,205))
                self.savedSet.SetValue("Config saved")
                formatXML(path)
        elif self.config=={} :
            dlg=wx.MessageDialog(self,
                                 'There are no reference created or loaded',
                                 'Message', wx.OK|wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return True
        elif self.comp==[] :
            dlg=wx.MessageDialog(self,
                                 'There are no comparison created or loaded',
                                 'Message', wx.OK|wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return True
        return True
#
    def ParseDom(self) :
        for param in self.dom.getElementsByTagName("params") :
            for node in param.childNodes :
                print(node)
                if node.nodeType!=1 :
                    continue
                self.config[node.nodeName]=node.firstChild.data
        for c in self.dom.getElementsByTagName("comparaison") :
            buf={}
            for node in c.childNodes :
                if node.nodeType==1 :
                    buf[node.nodeName]=node.firstChild.data.strip()
            self.comp.append(buf)
        self.SetRef()
        self.SetComp()
#        
    def SetRef(self) :
        try :
            self.refdir.SetValue(self.config["dir_ref"])
        except :
            self.refdir.SetValue("")
        try :
            self.refrec.SetValue(self.config["file_ref"])
        except :
            self.refrec.SetValue("gz****")
        try :
            self.sigma.SetValue(self.config["sigma"])
        except :
            self.sigma.SetValue("3")
        try :
            self.ordre.SetValue(self.config["ordre"])
        except :
            self.ordre.SetValue("1")
        try :
            self.cgtts.SetValue(self.config["cgt_type"])
        except :
            self.cgtts.SetValue("CGGTTS4")
        try :
            self.elvfil.SetValue(self.config["elvref"])
        except :
            self.elvfil.SetValue("0")
#
    def SetComp(self) :
        try :
            err=0
            self.lc.DeleteAllItems()
            for c in self.comp :
                num_items = self.lc.GetItemCount()
                try :
                    self.lc.InsertStringItem(num_items,c["labo"])
                except :
                    self.lc.InsertStringItem(num_items,"")
                try :
                    self.lc.SetStringItem(num_items,1,c["dir_comp"])
                except :
                    self.lc.SetStringItem(num_items,1,"")
                try :
                    self.lc.SetStringItem(num_items,2,c["file_comp"])
                except :
                    self.lc.SetStringItem(num_items,2,"")
                try :
                    self.lc.SetStringItem(num_items,3,c["dir_out"])
                except :
                    self.lc.SetStringItem(num_items,3,"")
                try :
                    self.lc.SetStringItem(num_items,4,c["file_out"])
                except :
                    self.lc.SetStringItem(num_items,4,"")
                try :
                    self.lc.SetStringItem(num_items,5,c["elvcom"])
                except :
                    self.lc.SetStringItem(num_items,5,"")
                try :
                    self.lc.SetStringItem(num_items,6,c["cgt_type_comp"])
                except :
                    self.lc.SetStringItem(num_items,6,"")
            return err
        except KeyError :
            err=-1
            return err
#
    def OnAddComp(self,event) :
        self.sb.SetStatusText("Adding comparisons")
        addframe=AddCompDial(None,-1,"Add comparison")
        addframe.Show(True)
        comploc=dict(compglo)
        self.comp.append(comploc)
        self.SetComp()
        self.savedSet.SetForegroundColour((255,255,230))
        self.savedSet.SetBackgroundColour((255,000,000))
        self.savedSet.SetValue("Config modified")
        return True
#
    def OnRemoveComp(self,event) :
        self.sb.SetStatusText("Removing comparisons")
        if len(self.comp)>=1 :
            dlg=wx.TextEntryDialog(self,'Name of the comparison to remove',
                                   'Get removed comp')
            dlg.SetValue("Default")
            if dlg.ShowModal()==wx.ID_OK :
                self.laboremoved=dlg.GetValue()
            dlg.Destroy
            for c in self.comp :
                if c["labo"]==self.laboremoved :
                    self.comp.remove(c)
        else :
            dlg=wx.MessageDialog(self,
                                 'There are no comparisons created or loaded',
                                 'Message', wx.OK|wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return True
        self.SetComp()
        self.savedSet.SetForegroundColour((255,255,230))
        self.savedSet.SetBackgroundColour((255,000,000))
        self.savedSet.SetValue("Config modified")
        return True
#
    def OnRemoveCompRC(self,event) :
        for c in self.comp :
            if c["labo"]==self.loboselected :
               self.comp.remove(c)
               self.SetComp()
               self.savedSet.SetForegroundColour((255,255,230))
               self.savedSet.SetBackgroundColour((255,000,000))
               self.savedSet.SetValue("Config modified")
               return True
#
    def OnModifieCompByDC(self,event) :
        try :
            self.labomodified=event.GetText()
        except AttributeError :
            self.labomodified=self.loboselected
        for c in self.comp :
            if c["labo"]==self.labomodified :
                modframe=ModifieCompDial(None,-1,"Edit comparison",c)
                modframe.Show(True)
                comploc=dict(compglo)
                self.comp.remove(c)
                self.comp.append(comploc)
                self.SetComp()
                self.savedSet.SetForegroundColour((255,255,230))
                self.savedSet.SetBackgroundColour((255,000,000))
                self.savedSet.SetValue("Config modified")
                return True
#
    def OnValidRef(self,event) :
        self.config["dir_ref"]=self.refdir.GetValue()
        try :
            if (self.config["dir_ref"][-1]!=SLASH) :
                self.config["dir_ref"]+=SLASH
        except IndexError :
            self.config["dir_ref"]=''
        self.config["file_ref"]=self.refrec.GetValue()
        self.config["sigma"]=self.sigma.GetValue()
        self.config["ordre"]=self.ordre.GetValue()
        self.config["elvref"]=self.elvfil.GetValue()
        self.config["cgt_type"]=self.cgtts.GetValue()
#
    def MakeOptions(self,boolstop,nstop,boolwsat,boolmad,boolepoch) :
        self.options=""
        if boolstop :
            if self.options=="" :
                self.options+="stop=%i"%(nstop)
            else :
                self.options+=",stop=%i"%(nstop)
        if boolwsat :
            if self.options=="" :
                self.options+="wsat"
            else :
                self.options+=",wsat"
        if boolmad :
            if self.options=="" :
                 self.options+="mad"
            else :
                 self.options+=",mad"
        if boolepoch :
            if self.options=="" :
                 self.options+="epoque"
            else :
                 self.options+=",epoque"
#
    def RunComp(self,event) :
        self.sb.SetStatusText("Running comparisons... please wait")
        # Argument pour le soft de comparaison
        if self.pathconfig == None :
            dlg=wx.MessageDialog(self,
                                 'There are no comparisons created or loaded',
                             'Message', wx.OK|wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return
        boolstop=self.bxstop.GetValue()
        nstop=self.bcnstop.GetValue()
        boolwsat=self.bxwsat.GetValue()
        boolmad=self.bxmad.GetValue()
        boolepoch=self.bxepoch.GetValue()
        boollog=self.bxelog.GetValue()
        self.MakeOptions(boolstop,nstop,boolwsat,boolmad,boolepoch)
        self.log.Clear()
        deb=self.bdeb.GetValue()
        fin=self.bfin.GetValue()
        print ("#\n# Launch soft GNSS CV time transfer\n#")
        print ("# Begin : %5i"%(int(deb)))
        print ("# End   : %5i"%(int(fin)))
        self.gauge.SetRange(fin-deb+1)
        for mjd in range(deb,fin+1) :
            argv=['main',self.pathconfig,mjd,self.options]
            cpr.main(argv)
            self.gauge.SetValue(mjd-deb+1)
            wx.SafeYield()
        self.sb.SetStatusText("Terminate comparisons succesfull")
        if boollog :
            saveout=self.log.GetValue().encode('utf-8')
            (dirlog,filelog)=os.path.split(self.pathconfig)
            if not os.path.exists(dirlog) :
                os.makedirs(dirlog)
            logoutpath=dirlog+"/%s_%i_%i.log"%(self.config["file_ref"],int(deb),int(fin))
            f=open(logoutpath,'w')
            f.write(saveout)
            f.close()
#
    def OnQuit(self,event) :
        self.Destroy()
#
class ViewPng(wx.Frame,object) :
    def __init__(self,parent,id,title,lopng) :
        wx.Frame.__init__(self,parent,id,title,size=(1000,620))
        panel=wx.Panel(self,-1,(20,20),(960,540))
        self.picture=wx.StaticBitmap(panel)
        self.lopng=lopng
        lfchoice=[]
        try :
            for png in self.lopng :
                lfchoice.append(png.split(SLASH)[-1])
            self.picture.SetFocus()
            self.picture.SetBitmap(wx.Bitmap(self.lopng[0]))
            wx.ComboBox(self,-1,lfchoice[0],pos=(20,580),size=(200,-1),
                        choices=lfchoice,style=wx.CB_READONLY)
        except TypeError :
            lfchoice=[]
            wx.ComboBox(self,-1,pos=(20,580),size=(200,-1),choices=lfchoice,
                        style=wx.CB_READONLY)
        self.Bind(wx.EVT_COMBOBOX,self.OnSelect)
#
    def OnSelect(self,event) :
        item=event.GetSelection()
        self.picture.SetFocus()
        self.picture.SetBitmap(wx.Bitmap(self.lopng[item]))
#
class AddCompDial(wx.Dialog):
    def __init__(self,parent,id,title):
        wx.Dialog.__init__(self,parent,id,title,size=(700,290))
#
        # Parametres comparaison
        # Texte des «boites»
        wx.StaticText(self,-1,'Name',(20,40))
        wx.StaticText(self,-1,'Comp directory',(20,80))
        wx.StaticText(self,-1,'Comp receiver',(20,120))
        wx.StaticText(self,-1,'Format',(295,120))
        wx.StaticText(self,-1,'Out directory',(20,160))
        wx.StaticText(self,-1,'Out file',(20,200))
        wx.StaticText(self,-1,'Elv filt',(400,200))
        # Def des «boites»
        self.Labo=wx.TextCtrl(self,-1,'',(135,35),(120,-1))
        self.compdir=wx.TextCtrl(self,-1,'',(135,75),(400,-1))
        self.comprec=wx.TextCtrl(self,-1,'gz****',(135,115),(120,-1))
        self.cgttscomp=wx.ComboBox(self,-1,'CGGTTS4',(355,115),(100,-1),
                                choices=['CGGTTS4','gps','gal','glo','mix'])
        self.outdir=wx.TextCtrl(self,-1,'',(135,155),(400,-1))
        self.outrec=wx.TextCtrl(self,-1,'',(135,195),(240,-1))
        self.outelv=wx.TextCtrl(self,-1,'0',(450,195),(60,-1))
#        
        # Bouton Browse comp
        broRef=wx.Button(self,92,'Browse',(545,70))
        self.Bind(wx.EVT_BUTTON,self.SearchDirComp,id=92)
        # Bouton Browse out
        broRef=wx.Button(self,93,'Browse',(545,150))
        self.Bind(wx.EVT_BUTTON,self.SearchDirOut,id=93)
        # Bouton Valid
        wx.Button(self,91,'&Valid',(20,235),(60,30))
        self.Bind(wx.EVT_BUTTON,self.OnValidComp,id=91)
        # Bouton cancel
        wx.Button(self,94,'&Cancel',(100,235),(80,30))
        self.Bind(wx.EVT_BUTTON,self.OnCancelComp,id=94)
#
        self.Centre()
        self.ShowModal()
        self.Destroy()
#
    def SearchDirComp(self,event) :
        p=ID_PATH
        dlg=wx.DirDialog(self, "Choose a directory",
                           defaultPath=p, 
                           style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if dlg.ShowModal() == wx.ID_OK:
            self.CompDirV=dlg.GetPath()
            self.compdir.SetValue(self.CompDirV+SLASH)
        dlg.Destroy()
#
    def SearchDirOut(self,event) :
        p=ID_PATH
        dlg = wx.DirDialog(self,"Choose a directory",
                           defaultPath=p, 
                           style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if dlg.ShowModal() == wx.ID_OK:
            self.CompOutV=dlg.GetPath()
            self.outdir.SetValue(self.CompOutV+SLASH)
        dlg.Destroy()
#
    def OnValidComp(self,event):
        if self.Labo.GetValue()!="" :
            compglo["labo"]=self.Labo.GetValue()
            compglo["dir_comp"]=self.compdir.GetValue()
            compglo["file_comp"]=self.comprec.GetValue()
            compglo["cgt_type_comp"]=self.cgttscomp.GetValue()
            compglo["dir_out"]=self.outdir.GetValue()
            compglo["file_out"]=self.outrec.GetValue()
            compglo["elvcom"]=self.outelv.GetValue()
            if (compglo["dir_comp"][-1]!=SLASH) :
                compglo["dir_comp"]+=SLASH
            if (compglo["dir_out"][-1]!=SLASH) :
                compglo["dir_out"]+=SLASH
        else :
            dlg=wx.MessageDialog(self,'Give a name',
                                 'Message', wx.OK|wx.ICON_INFORMATION)
        self.Destroy()
#
    def OnCancelComp(self,event) :
        self.Destroy()
#
class ModifieCompDial(wx.Dialog,object) :
    def __init__(self,parent,id,title,c):
        wx.Dialog.__init__(self,parent,id,title,size=(700,290))
#
        # Parametres comparaison
        # Texte des «boites»
        wx.StaticText(self,-1,'Name',(20,40))
        wx.StaticText(self,-1,'Comp directory',(20,80))
        wx.StaticText(self,-1,'Comp receiver',(20,120))
        wx.StaticText(self,-1,'Format',(295,120))
        wx.StaticText(self,-1,'Out directory',(20,160))
        wx.StaticText(self,-1,'Out file',(20,200))
        wx.StaticText(self,-1,'Elv filt',(400,200))
        # Def des «boites»
        try :
            self.Labo=wx.TextCtrl(self,-1,c["labo"],(135,35),(120,-1))
        except KeyError :
            self.Labo=wx.TextCtrl(self,-1,"",(135,35),(120,-1))
        try :
            self.compdir=wx.TextCtrl(self,-1,c["dir_comp"],(135,75),(400,-1))
        except KeyError :
            self.compdir=wx.TextCtrl(self,-1,"",(135,75),(400,-1))
        try :
            self.comprec=wx.TextCtrl(self,-1,c["file_comp"],(135,115),(120,-1))
        except KeyError :
            self.comprec=wx.TextCtrl(self,-1,"",(135,115),(120,-1))
        try :
            self.cgttscomp=wx.ComboBox(self,-1,c["cgt_type_comp"],(355,115),
                                       (100,-1),choices=['CGGTTS4','gps',
                                                         'glo','mix'])
        except KeyError :
            self.cgttscomp=wx.ComboBox(self,-1,'CGGTTS4',(355,115),(100,-1),
                                       choices=['CGGTTS4','gps','glo','mix'])
        try :
            self.outdir=wx.TextCtrl(self,-1,c["dir_out"],(135,155),(400,-1))
        except KeyError :
            self.outdir=wx.TextCtrl(self,-1,"",(135,155),(400,-1))
        try :
            self.outrec=wx.TextCtrl(self,-1,c["file_out"],(135,195),(240,-1))
        except KeyError :
            self.outrec=wx.TextCtrl(self,-1,"",(135,195),(240,-1))
        try :
            self.outelv=wx.TextCtrl(self,-1,c["elvcom"],(450,195),(60,-1))
        except KeyError :
            self.outelv=wx.TextCtrl(self,-1,"",(450,195),(60,-1))
#        
        # Bouton Browse comp
        broRef=wx.Button(self,92,'Browse',(545,70))
        self.Bind(wx.EVT_BUTTON,self.SearchDirComp,id=92)
        # Bouton Browse out
        broRef=wx.Button(self,93,'Browse',(545,150))
        self.Bind(wx.EVT_BUTTON,self.SearchDirOut,id=93)
        # Bouton Valid
        wx.Button(self,91,'&Valid',(20,235),(60,30))
        self.Bind(wx.EVT_BUTTON,self.OnValidComp,id=91)
        # Bouton cancel
        wx.Button(self,94,'&Cancel',(100,235),(80,30))
        self.Bind(wx.EVT_BUTTON,self.OnCancelComp,id=94)
#
        # Sauvegarde pour le bouton Cancel
        self.comptemp=c
#
        self.Centre()
        self.ShowModal()
        self.Destroy()
#
    def SearchDirComp(self,event) :
        p=ID_PATH
        dlg = wx.DirDialog(self, "Choose a directory",
                           defaultPath=p, 
                           style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if dlg.ShowModal() == wx.ID_OK:
            self.CompDirV=dlg.GetPath()
            self.compdir.SetValue(self.CompDirV+SLASH)
        dlg.Destroy()
#
    def SearchDirOut(self,event) :
        p=ID_PATH
        dlg = wx.DirDialog(self,"Choose a directory",
                           defaultPath=p, 
                           style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if dlg.ShowModal() == wx.ID_OK:
            self.CompOutV=dlg.GetPath()
            self.outdir.SetValue(self.CompOutV+SLASH)
        dlg.Destroy()
#
    def OnValidComp(self,event):
        if self.Labo.GetValue()!="" :
            compglo["labo"]=self.Labo.GetValue()
            compglo["dir_comp"]=self.compdir.GetValue()
            compglo["file_comp"]=self.comprec.GetValue()
            compglo["cgt_type_comp"]=self.cgttscomp.GetValue()
            compglo["dir_out"]=self.outdir.GetValue()
            compglo["file_out"]=self.outrec.GetValue()
            compglo["elvcom"]=self.outelv.GetValue()
            if (compglo["dir_comp"][-1]!=SLASH) :
                compglo["dir_comp"]+=SLASH
            if (compglo["dir_out"][-1]!=SLASH) :
                compglo["dir_out"]+=SLASH
        else :
            dlg=wx.MessageDialog(self,'Give a name',
                                 'Message', wx.OK|wx.ICON_INFORMATION)
        self.Destroy()
        return
#
    def OnCancelComp(self,event) :
        self.Destroy()
        return
#
#
class RedirectText(object):
    def __init__(self,aWxTextCtrl):
        self.out=aWxTextCtrl
#
    def write(self,string):
        wx.CallAfter(self.out.WriteText,string)
#
class MainApp(wx.App) :
    def OnInit(self) :
        print("Systeme d'exploitation : %s"%(platform.system()))
        frame=MainFrame(None,-1,"lcComView")
        frame.Show(True)
        return True
#
# MAIN
if __name__ == "__main__" :
    app=MainApp()
    app.MainLoop()
