# encoding: utf8
#
"""
Copyright or © or Copr. Baptiste Chupin (2013-07-15)

baptiste.chupin@obspm.fr

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author, the holder of the
economic rights, and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading, using, modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate, and  that  also
therefore means  that it is reserved for developers and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and, more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

$Id: load_config.py 572 2021-04-14 13:28:32Z chupin $
"""
#
import sys
import xml.dom.minidom
#
def load(configfile) :
    try :
        dom=xml.dom.minidom.parse(configfile)
    except IOError :
        print ("File not found : %s " % (configfile))
        sys.exit(1)
    config={}
    for param in dom.getElementsByTagName("params") :
        for node in param.childNodes :
            if node.nodeType!=1 :
                continue
            config[node.nodeName]=node.firstChild.data

    comp=[]
    for c in dom.getElementsByTagName("comparaison") :
        buf={}
        for node in c.childNodes :
            if node.nodeType==1 :
                buf[node.nodeName]=node.firstChild.data.strip()
        comp.append(buf)

    etal=[]
    for c in dom.getElementsByTagName("etalonnage") :
        buf={}
        for node in c.childNodes :
            if node.nodeType==1 :
                buf[node.nodeName]=node.firstChild.data.strip()
        buf['etal']='yes'
        etal.append(buf)
        
    if comp != [] :
        return config,comp
    elif etal != [] :
        return config,etal
#
def mkFilename(path,recName,mjd,cgtType) :
    CGtype5=['gps','gal','glo','mix','C1','P1','P2']
    if(cgtType=='CGGTTS4') :
        filename=path+recName+("%6.3f"%(int(mjd)/1000.))
    elif (cgtType in CGtype5) :
        #filename=path+recName+("%5i."%(int(mjd)))+cgtType
        if (cgtType not in ['C1','P1','P2','L3P']):
            filename="%s/%6s%5i.%3s"%(path,recName,int(mjd),cgtType)
        else :
            filename="%s/%6s%5i.gps"%(path,recName,int(mjd))
    else :
        print("# Mauvaise definition du Type de CGGTTS \
              dans le fichier de config !")
        filename=None
    return filename
