#!/usr/bin/python3
# encoding: utf8
#
"""
Copyright SYRTE -- Observatoire de Paris
contributor(s) :
Baptiste Chupin (2013-07-15) - baptiste.chupin@obspm.fr

This software is a computer program whose purpose is to improve the short term 
stability and the robustness of Common-View Global Position System satellites.

This software is governed by the CeCILL-B license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author, the holder of the
economic rights, and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading, using, modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate, and  that  also
therefore means  that it is reserved for developers and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and, more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

Auteur  : B. Chupin
Date    : 26-06-2012
Version : 3.0

Version 2   : Mise en place d'une interface graphique pour mettre en route les comparaisons.
Version 2.1 : Mise en place de l'option pour les version CGTTS (4 ou 5)
Version 2.2 : Ajout de l'élévation min pour garder les satellites 
Version 3.0 : Gestion des fichiers 30s

$Id: main.py 588 2021-10-05 11:53:16Z chupin $
"""
# modules python
import os,sys
import datetime
import math
import timeconv
# modules locaux
import load_config as lc
import write_out as wo
import comview as cv
import usage
#
def main(argv) :
    #print ("# %s"%sys.version[0])
    # Definition du repertoire de travail
    w_dir=""
    # Definitions :
    now=datetime.datetime.now() # objet datetime aujourd'hui
    yy=int(now.year)
    mm=int(now.month)
    dd=int(now.day)
    hh=int(now.hour)
    mn=int(now.minute)
    ss=int(now.second)
    arguments=""
    # Date à traiter et gestion des arguments
    if len(argv) < 2 :
        usage.usage()
    try :
        if argv[1]=="--help" :
            usage.usage()
        else :
            configfile=argv[1]
            try :
                mjd=float(argv[2])
                # Gestion arguments
                try :
                    arguments=argv[3]
                except IndexError :
                    arguments=""
            except ValueError :
                arguments=argv[2]
                dt=now-datetime.timedelta(days=1)
                mjd=timeconv.mjd(dt,'float')
    except IndexError :
        dt=now-datetime.timedelta(days=1)
        mjd=timeconv.mjd(dt,'float')
    print("#\n#    Execution du script %s "%(argv[0]))
    print("#    Le %02i/%02i/%4i "%(dd,mm,yy))
    ajrdh=timeconv.mjd2utc(mjd,'calen')
    print("#    Traitement du jour %2s/%2s/%4s  \n#"
          %(ajrdh[8:10],ajrdh[5:7],ajrdh[0:4]))
    #
    yyyy=timeconv.mjd2utc(mjd,'yearonly')
    # Mise à jour des flags en fonctions des arguments    
    flag_mad,flag_filt_epo,flag_wsat,flag_stop,flag_insert,flag_etal,flag_30s,nstop,code=usage.options(arguments)
    # Definition du fichier de config en fonction des arguments
    print("# Fichier de config : %s"%(configfile))
    # Lecture du fichier de config et récuperation des paramtres 
    # (dans le dictionnaire config) et des comparaisons (dans la liste comp)
    config,comp=lc.load(configfile)
    ordre=int(config["ordre"])  # Ordre du fit pour filtrage
    ns   =int(config["sigma"])  # filtre à ns sigma
    try :
        elvm=int(config["elvref"])*10    # filtre sur l'élévation du satelite
    except KeyError :
        elvm=0
    try : 
        cggVer=config["cgt_type"]
    except KeyError :
        cggVer='CGGTTS4'
    # Lecture du fichier reférence
    refname=lc.mkFilename(config["dir_ref"],
                          config["file_ref"],
                          mjd,config["cgt_type"])
    if ("[YYYY]" in refname) :
        refname=refname.replace("[YYYY]",yyyy)
    if os.path.exists(refname) :
        print("# Fichier de reference : %s"%(refname))
        if (cggVer=='C1'):
            opprn,opmjd,opelv,opazth,oprefgps=cv.extract_cggtts(refname,cggVer,elvm,'C1')
        else:
            if(flag_30s):
                opprn,opmjd,opelv,opazth,oprefgps=cv.extract_cggtts_30s(refname,cggVer,elvm,code)
            else:
                opprn,opmjd,opelv,opazth,oprefgps=cv.extract_cggtts(refname,cggVer,elvm)
        oprpt=cv.makedicrpt(opprn,opmjd,opelv,opazth,oprefgps)
    else :
        print("# Le fichier reference \n# %s \n# n'existe pas"%(refname))
        return
    for xc in comp : # boucle sur le flag <comparaison> du fichier de config xml
            try :
                elvm=int(xc["elvcom"])*10
            except KeyError :
                elvm=0
            try : 
                cggVer=xc["cgt_type_comp"]
            except KeyError :
                cggVer='CGGTTS4'
            dir_out=xc["dir_out"]
            if ("[YYYY]" in dir_out) :
                dir_out=dir_out.replace("[YYYY]",yyyy)
            fil_out=xc["file_out"]
#            filename=xc["dir_comp"]+xc["file_comp"]+\
#                        "%6.3f"%(int(mjd)/1000.) # Le deuxieme terme c'est le 
                                                 # MJD 'dotted'
            filename=lc.mkFilename(xc["dir_comp"],xc["file_comp"],
                                   mjd,xc["cgt_type_comp"])
            if ("[YYYY]" in filename) :
                filename=filename.replace("[YYYY]",yyyy)
            numlab=xc["labo"] # Le numero de Labo est renseigné dans le 
                              # fichier de config xml
            print("#\n# Traitement : %s\n#"%numlab)
            if os.path.exists(filename) :# Verification de l'existance du 
                                         # fichier GPS 
                print("# Traitement du fichier %s"%(filename))
                if (filename != refname) :
                    if (cggVer=='C1'):
                        cpprn,cpmjd,cpelv,cpazth,cprefgps=\
                            cv.extract_cggtts(filename,cggVer,elvm,'C1')
                    else :
                        if(flag_30s):
                            cpprn,cpmjd,cpelv,cpazth,cprefgps=\
                                    cv.extract_cggtts_30s(filename,cggVer,elvm,code)
                        else:
                            cpprn,cpmjd,cpelv,cpazth,cprefgps=\
                            cv.extract_cggtts(filename,cggVer,elvm)
                    cprpt=cv.makedicrpt(cpprn,cpmjd,cpelv,cpazth,cprefgps)
                    diff=cv.makecomprpt(oprpt,cprpt) # Fabricat° du dictionnaire
                                                     # cprpt-oprpt pour chaque 
                                                     # sat
                else :
                    diff=cv.makealone(oprpt)
                if len(diff) == 0 :
                    print("# Le fichier\n# %s\n# est vide"%(filename))
                    continue
                if flag_mad :
                    flag_sort=False # Si le filtre est de type MAD, 
                                    # il n'y a pas de type sigma
                else :
                    flag_sort=True
                    ntxcl=0
                nbcle=0
                if flag_wsat :
                    wo.savesat(diff,dir_out+"/sat/before/")
                while flag_sort : # Flag pour tester la convergeance du filtre
                    nbcle+=1
                    p=cv.fit_day(diff,ordre) # Fit d'ordre n sur une journée
                    diff,flag_sort,nxcl=cv.clean_diff(diff,p,ns) # Filtre à N 
                                                                 # sigma (ns)
                    ntxcl+=nxcl
                    if (flag_stop) and (nbcle==nstop) :
                        break
                while flag_mad :
                    nbcle+=1
                    diff,ntxcl=cv.mad_diff(diff,ns)
                    flag_mad=False
                print("# Nombre de satellites retirés : %02i"%(ntxcl))
                print("# Nombre de boucles de filtrage : %02i"%(nbcle))
                if flag_wsat :
                    wo.savesat(diff,dir_out+"/sat/after/")
                epoque=cv.mean_epoque(diff) # Moyennage par époque d'obs
                if flag_filt_epo : # si flag_filt_epo=True on filtre aussi 
                                   # les époques
                    ntepoxcl=0
                    flag_sort=True
                    while flag_sort :
                        epoque,flag_sort,nepoxcl=cv.filt_epoque(epoque,ordre)
                        ntepoxcl+=nepoxcl
                    print("# Nombre d'époque retirées : %02i"%(ntepoxcl))
                #mean_day=cv.extra_epoques(epoque,ordre,mjd)
                try :
                    fo=wo.save_link(epoque,dir_out,fil_out,flag_insert)
                except IndexError :
                    pass
                # Sauvergarde des différences (append, insert ou replace)
            else :
                print("# Le fichier \n# %s \n# n'existe pas"%(filename))
                continue
    return
    #
    #sys.exit(0)
#
if __name__=='__main__' :
    main(sys.argv)
    sys.exit(0)
