#!/usr/bin/env python
# encoding: utf8

""" Module de convertion des dates (MJD,Calendaire, etc)
$Id: timeconv.py 493 2017-09-13 15:25:36Z chupin $
"""

import sys
import datetime
import math

""" Transforme une date MJD en date calendaire
    Par F.meynadier 2011-11-17
"""
def mjd2utc(MJD,depth="calen",sep='-') :
   sep=sep[0]
   JD = 2400000.5 + MJD
   Z = math.floor(JD+0.5)
   F = JD+0.5 - Z
   if (Z < 2299161):
       A = Z
   else:
       I = math.floor((Z - 1867216.25)/36524.25);
       A = Z + 1 + I - math.floor(I/4.);

   B = A + 1524
   C = math.floor((B - 122.1)/365.25)
   D = math.floor(365.25 * C)
   T = math.floor((B - D)/ 30.6001)
   RJ = B - D - math.floor(30.6001 * T) + F
   JJ = math.floor(RJ);
   RH = (RJ - math.floor(RJ)) * 24;
   Heure = math.floor(RH);
   Mn = math.floor((RH - Heure )*60);
   Sec = ((RH - Heure )*60 - Mn )*60;
   if (T < 14) :
       MM = T - 1
   else :
       if ((T == 14) or (T == 15)):
           MM = T - 13

   if (MM > 2) :
       AA = C - 4716
   else :
       if ((MM == 1) or (MM == 2)):
           AA = C - 4715

   if depth == "full":
       return("%04d%1s%02d%1s%02d %02d:%02d:%02d" % (AA,sep,MM,sep,JJ, Heure, Mn, Sec))
   elif depth == "calen":
       return("%04d%1s%02d%1s%02d"%(AA,sep,MM,sep,JJ))
   elif depth == "calenFre":
       return("%02d%1s%02d%1s%04d"%(JJ,sep,MM,sep,AA))
   elif depth == "calenFreFull":
       return("%02d%1s%02d%1s%04d %02d:%02d:%02d"%(JJ,sep,MM,sep,AA,Heure, Mn, Sec))
   elif depth == "yearonly":
       return("%04d" % (AA))

# Transforme un objet datetime.datetime en MJD
# Par F. Meynadier
# 
def mjd(dt,flag="int"):
    YY = dt.year
    MM = dt.month
    DD = dt.day
    HR = dt.hour
    MN = dt.minute
    SS = dt.second

    HR = HR + (MN / 60.) + (SS / 3600.)
    GGG = 1
    if (YY < 1582) :
        GGG = 0
    if (YY <= 1582 and MM < 10):
         GGG = 0
    if (YY <= 1582 and MM == 10 and DD < 5):
        GGG = 0
    JD = -1 * math.floor(7 * (math.floor((MM + 9) / 12.) + YY) / 4.)
    S = 1;
    if ((MM - 9)<0):
        S=-1
    A = math.fabs(MM - 9)
    J1 = math.floor(YY + S * math.floor(A / 7))
    J1 = -1 * math.floor((math.floor(J1 / 100) + 1) * 3 / 4.)
    JD = JD + math.floor(275 * MM / 9.) + DD + (GGG * J1)
    JD = JD + 1721027 + 2 * GGG + 367 * YY - 0.5
    JD = JD + (HR / 24.)
    MJD = JD - 2400000.5

    if flag=="string" :
        return "%6f" % (int(MJD))
    if flag=="float" :
        return MJD
    if flag=="int" :
        return int(MJD)
    if flag=="dotted" :
        return "%6.3f" % (int(MJD)/1000.)

def mjd2(YY,MM,DD,HR,MN,SS,flag="int"):
    
    HR = HR + (MN / 60.) + (SS / 3600.)
    GGG = 1
    if (YY < 1582) :
        GGG = 0
    if (YY <= 1582 and MM < 10):
         GGG = 0
    if (YY <= 1582 and MM == 10 and DD < 5):
        GGG = 0
    JD = -1 * math.floor(7 * (math.floor((MM + 9) / 12.) + YY) / 4.)
    S = 1;
    if ((MM - 9)<0):
        S=-1
    A = math.fabs(MM - 9)
    J1 = math.floor(YY + S * math.floor(A / 7))
    J1 = -1 * math.floor((math.floor(J1 / 100) + 1) * 3 / 4.)
    JD = JD + math.floor(275 * MM / 9.) + DD + (GGG * J1)
    JD = JD + 1721027 + 2 * GGG + 367 * YY - 0.5
    JD = JD + (HR / 24.)
    MJD = JD - 2400000.5

    if flag=="string" :
        return "%6f" % (int(MJD))
    if flag=="float" :
        return MJD
    if flag=="int" :
        return int(MJD)
    if flag=="dotted" :
        return "%6.3f" % (int(MJD)/1000.)

def convMjdCal(imjd) :
    J=imjd+2400000+32044+1
    g=J/146097
    dg=J%146097
    c=dg/36524
    if (c==4) :
        c-=1
    c=((dg/36524+1)*3)/4
    dc=dg-c*36524
    b=dc/1461
    db=dc%1461
    a=db/365
    a=((db/365+1)*3)/4
    da=db-a*365
    y=400*g+100*c+4*b+a
    m=(da*5+308)/153-2
    d=da-(m+4)*153/5+122
    yy=y-4800+(m+2)/12
    mm=(m+2)%12+1-1
    dd=d+1
    doy=(da+59+db/1460)%365+1
    if(a==0)and(da<306)and((yy%100)or(yy%400)) :
        doy=(da+59)%365+2
    else :
        doy=(da+59)%365+1
    dow=1+((J+2)%7)
    return dd,mm,yy,dow,doy

def h2j(h,m,s) :
    dj=(1./24.)*((1./60.)*(s/60.+m)+h)
    return dj

if __name__ == "__main__" :
    now=datetime.datetime.now()
    yy=int(now.year)
    mm=int(now.month)
    dd=int(now.day)
    hh=int(now.hour)
    mn=int(now.minute)
    ss=int(now.second)
    dt=now-datetime.timedelta(days=0)
    enter=[]
    for i in range(1,len(sys.argv)) :
        enter.append(sys.argv[i])
    if (len(enter)==0) :
        mjdPrint=mjd(dt,'float')
        print(mjdPrint)
    elif (len(enter)==1) :
        try :
            mjdRead=float(enter[0])
            datePrint=mjd2utc(mjdRead,"calenFreFull",sep=" ")
            print(datePrint)
        except ValueError :
            print((mjd(dt,'float')))
    elif (len(enter)==2) :
        J=int(math.floor(float(enter[1])))
        dmjd=float(enter[1])-J
        dd,mm,yy,dow,doy=convMjdCal(J)
        if (enter[0][-1]=='d') :
            print(doy)
        elif (enter[0][-1]=='w') :
            print(dow)
        elif (enter[0][-1]=='a') :
            print(dd,mm,yy,dow,doy)
        else :
            print("option %s non reconnu"%(enter[0]))

    else :
        try : 
            dd=int(enter[0])
        except IndexError :
            dd=dd
        except ValueError :
            dd=0o1
        try : 
            mm=int(enter[1])
        except IndexError :
            mm=mm
        except ValueError :
            mm=0o1
        try : 
            yy=int(enter[2])
        except IndexError :
            yy=yy
        except ValueError :
            yy=yy
        try : 
            hh=int(enter[3])
        except IndexError :
            hh=0
        try : 
            mn=int(enter[4])
        except IndexError :
            mn=0
        try : 
            ss=int(enter[5])
        except IndexError :
            ss=0
        dateRead=dt.replace(yy,mm,dd,hh,mn,ss)
        print((mjd(dateRead,'float')))
    sys.exit(0)
