# encoding: utf8

"""Copyright or © or Copr. Baptiste Chupin (2013-07-15)

baptiste.chupin@obspm.fr

This software is governed by the [CeCILL|CeCILL-B|CeCILL-C] license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the [CeCILL|CeCILL-B|CeCILL-C]
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author, the holder of the
economic rights, and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading, using, modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate, and  that  also
therefore means  that it is reserved for developers and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and, more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the [CeCILL|CeCILL-B|CeCILL-C] license and that you accept its terms.

$Id: usage.py 572 2021-04-14 13:28:32Z chupin $
"""
import re
import sys
def usage() :
    message="""
    Programme de traitement de vues communes entre deux recepteurs code P3
    à partir de fichiers au format CGGTTS.
    Le programme interroge un fichier de configuration xml (config.xml) 
    où est indiqué le recepteur de référence ainsi que les recepteurs comparés
    et les paramètres de traitement des données

    usage :

        python main.py configfile [date] [option1,option2...] (les options sont 
        séparées par une virgule sans espace)
        
            configfile :
                fichier de config au format xml

            date : 
                mjd à traiter, par defaut j-1
         
            options : 
                stop=N, permet de stoper la boucle de filtrage à l'itération N
                wsat, pour ecrire les fichiers satellites par satellites avant 
                      et apres filtrage
                mad, méthode de filtrage par une méthode MAD-based
                epoque, filtrage sur les moyennes par époque après le premier 
                        filtrage
                noinsert, le fichier de sortie est ouvert selement en append
                etalonnage=code, traite des fichiers 30s pour étalonner les retards P1 et P2 du recepteur comparé code =[P1|P2]
        
        python main.py --help 
            affiche ce message 
    """
    print(message)
    sys.exit(-1)

def options(args) :
    flag_filt_epo=False # Flag pour effectuer un filtre sur les moyennes par 
                        # époques abérantes
    flag_mad     =False # Flag pour switcher entre un filtre à N sigma et une 
                        # méthode MAD
    flag_stop    =False # Flag pour arreter la boucle de filtrage à N iterations
    flag_wsat    =False # Flag pour ecrire les fichiers satellites par 
                        # sattelites avant et apres filtrage
    flag_insert  =True  # Flag pour inserer les data dans fichiers de sortie
    flag_etal    =False # Flag pour faire un etalonnage sur le code
    flag_30s     =False # Falg pour traiter les fichiers 30s 
    code='L3'
    nstop=0
    loarg=args.split(',')
    if len(args) > 100 :
        usage()
    if ("mad" in loarg) :
        flag_mad=True
        print("# Filtrage par une méthode MAD-based")
    else :
        print("# Filtrage à N*sigma (defini dans le fichier de config xml)")
    if ("epoque" in loarg) :
        flag_filt_epo=True
    if ("wsat" in loarg) :
        flag_wsat=True
    if ("noinsert" in loarg) :
        flag_insert=False
    for l in loarg :
        if re.search('stop',l) != None :
            try :
                flag_stop=True
                nstop=int(l.split("=")[1])
            except IndexError :
                usage()
        if re.search('etalonnage',l) != None :
            try :
                flag_etal=True
                code=l.split("=")[1]
            except IndexError :
                usage()
        if re.search('30s',l) != None :
            try:
                flag_30s=True
                code=l.split("=")[1]
            except IndexError :
                usage()
#
    return flag_mad,flag_filt_epo,flag_wsat,flag_stop,flag_insert,flag_etal,\
            flag_30s, nstop,code
