# encoding: utf8
""" 
Copyright or © or Copr. Baptiste Chupin (2013-07-15)

baptiste.chupin@obspm.fr

This software is governed by the [CeCILL|CeCILL-B|CeCILL-C] license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the [CeCILL|CeCILL-B|CeCILL-C]
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author, the holder of the
economic rights, and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading, using, modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate, and  that  also
therefore means  that it is reserved for developers and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and, more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the [CeCILL|CeCILL-B|CeCILL-C] license and that you accept its terms.

Module d'enregistrement des calcul

$Id: write_out.py 572 2021-04-14 13:28:32Z chupin $

"""
# modules python
import os
import math
def save_link(mesure,dirout,fname,flag_insert=True) :
    """ 
        Module pour l'enregistrement des liaisons GPS
        rpt2-rpt1

        file=save_link(mesure,day,dirout,fname)

        INPUT  : 
            mesure[] : [mjd,diff,sig,np], liste des valeurs à sauver dans un 
                                          fichiers (append)
                       mjd   : float, date de la mesure
                       diff  : float, mesure moyenne à sauvegarder
                       sig   : float, ecart-type de la mesure moyenne
                       np    : int, nombre de points pour réaliser la moyenne
            day[]    : mjd,phi00,phi14
                       mjd   : float, mjd rond de la mesure
                       phi00 : float, extrapolation de la mesure à 00 h
                       phi14 : float, extrapolation de la mesure à 14 h
            dirout   : str, chemin du repertoire de sortie defini dans le 
                            config.xml
            fname    : str, fichier de sortie defini dans le config.xml
        OUTPUT :
            f        : <closed file u'chemin absolu', mode 'w'>
    """
#
    filename=dirout+"/"+fname
    print("# Fichier de sortie : %s"%filename)
    if(flag_insert) :
        if os.path.exists(filename) :
            le=len(mesure)
            f=open(filename,'r')
            data=f.readlines()
            f.close()
            n=len(data)
            mjd_deb=math.floor(mesure[0][0])
            mjd_fin=mjd_deb+1
            deb=0
            lomjd=[]
            tmp=data
            for d in tmp :
                if (float(d.split()[0])>=mjd_deb)and(float(d.split()[0])<=mjd_fin) :
                    err=data.remove(d)
            for d in data :
                lomjd.append(d.split()[0])
            for m in mesure :
                srch_str="%11.5f %11.3f %7.3f %02i\n"%(m[0],m[1],m[2],m[3])
                if srch_str not in data :
                    if "%11.5f"%(m[0]) not in lomjd :
                        data.append("%11.5f %11.3f %7.3f %02i\n"
                                    %(m[0],m[1],m[2],m[3]))
                    elif "%11.5f"%(m[0]) in lomjd :
                        ich=lomjd.index("%11.5f"%(m[0]))
                        data[ich]="%11.5f %11.3f %7.3f %02i\n"\
                                %(m[0],m[1],m[2],m[3])
            data.sort()
            f=open(filename,'w')
            f.writelines(data)
            f.close()
        else :
            f=open(filename,'w')
            for m in mesure :
                f.write("%11.5f %11.3f %7.3f %02i\n"%(m[0],m[1],m[2],m[3]))
            f.close
    else :
        f=open(filename,'a')
        for m in mesure :
            f.write("%11.5f %11.3f %7.3f %02i\n"%(m[0],m[1],m[2],m[3]))
        f .close
#   
    return f
#
def savesat(diff,dirout) :
    """
       Module pour l'enregistrement des fichiers satelite par satelite, 
       si le fichier existe l'écriture ce fait en append
       Attention les ne sons pas nettoyé, existance de doublons si le programme
       tourne plusieur fois pour le même jour

       Usage : file=write_out.savesat(diff,dirout) :

           INPUT : 
               diff{}   : 'key'         : str, N satellite commun au deux 
                                          recepteurs
                          diff['key'][] : [], pour chaque satellite :
                                          mjd,diff
               dirout   : str, chemin du repertoire de sortie defini dans le 
                               config.xml
           OUTPUT :
               f        : <closed file u'chemin absolu', mode 'w'>

    """
    if not os.path.exists(dirout) :
        os.makedirs(dirout)
    for k in diff.keys() :
        filename=dirout+k+".dat"
        if os.path.exists(filename) :
            f=open(filename,'a')
            for d in diff[k] :
                f.write("%12.5f %12.9f\n"%(d[0],d[1]))
            f.close()
        else :
            f=open(filename,'w')
            for d in diff[k] :
                f.write("%12.5f %12.9f\n"%(d[0],d[1]))
            f.close()
#
    return f
