# encoding: utf8
#
""" 
Copyright or © or Copr. Baptiste Chupin (2013-07-15)

baptiste.chupin@obspm.fr

This software is a computer program whose purpose is to is to improve the short term stability and the robustness of Common-View Global Position System satellites.

This software is governed by the [CeCILL|CeCILL-B|CeCILL-C] license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the [CeCILL|CeCILL-B|CeCILL-C]
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author, the holder of the
economic rights, and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading, using, modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate, and  that  also
therefore means  that it is reserved for developers and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and, more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the [CeCILL|CeCILL-B|CeCILL-C] license and that you accept its terms.

Module pour la gestion des comparaisons de recepteur
Méthode de vue commune avec filtre à N sigma sur une boucle while

$Id: comview.py 588 2021-10-05 11:53:16Z chupin $
"""
# modules python
import os
import numpy as np
from operator import itemgetter
# modules du path
import timeconv
#
def extract_cggtts(filename,cggVer,elvMin=150,bande='L3') :
    """ Routine pour extraire les données exploitées dans un fichier au format cggtts
    
    Usage : prn,mjd,elv,azth,refgps=comview.extract_cggtts(filename)
    
        INPUT  :
             filename : str,   fichier cggtts
             elvMin   : int,   filtre sur les elevations (0.1 deg) 
                               valeur par defaut 150
        OUTPUT :
             prn[]    : str,   N satellite
             mjd[]    : float, date MJD de la comparaison
             elv[]    : int,   elavation
             azth[]   : int,   azimuth
             refgps[] : float, difference entre la reference 
                               du recepteur et GPSTIME en ns
    """
    f=open(filename,'r')
    prn=[]
    mjd=[]
    trkl=[]
    elv=[]
    azth=[]
    refgps=[]
    for d in f.readlines() :
        if (cggVer=='CGGTTS4') :
            if (d[0] == 'G') :
                tmp=d[1:].split()
            else :
                tmp=d.split()
        else :
            tmp=d[1:].split()
        try : # Pour eliminer le header on essaye de «spliter» la ligne
              # si c'est pas possible ce n'est pas une ligne de données
            if (int(tmp[4]) == 780) and (int(tmp[5]) > elvMin)\
                    and (tmp[22][:2]==bande) : # Il faut que les poursuites est 
                                      #  durées 780 s pour être retenues
                prn_a="%02i"%(int(tmp[0]))
                prn.append(prn_a)
                mjd.append(float(tmp[2])+
                    timeconv.h2j(float(tmp[3][:2]),float(tmp[3][2:4]),
                                 float(tmp[3][4:6])))
                elv.append(int(tmp[5]))
                azth.append(int(tmp[6]))
                refgps.append(int(tmp[9])/10.)
        except ValueError :
            continue
        except IndexError :
            continue
    f.close()
#
    return prn,mjd,elv,azth,refgps # Retourne 5 listes
#
def extract_cggtts_30s(filename,cggVer,elvMin=150,bande="L3") :
    """ Routine pour extraire les données exploitées dans un fichier au format cggtts
    
    Usage : prn,mjd,elv,azth,refgps=comview.extract_cggtts(filename)
    
        INPUT  :
             filename : str,   fichier cggtts
             elvMin   : int,   filtre sur les elevations (0.1 deg) 
                               valeur par defaut 150
        OUTPUT :
             prn[]    : str,   N satellite
             mjd[]    : float, date MJD de la comparaison
             elv[]    : int,   elavation
             azth[]   : int,   azimuth
             refgps[] : float, difference entre la reference 
                               du recepteur et GPSTIME en ns
    """
    if bande == "C1":
        bande = "L1C"
    f=open(filename,'r')
    prn=[]
    mjd=[]
    trkl=[]
    elv=[]
    azth=[]
    refgps=[]
    for d in f.readlines() :
        tmp=d[1:].split()
        try : # Pour eliminer le header on essaye de «spliter» la ligne
              # si c'est pas possible ce n'est pas une ligne de données
            if (int(tmp[3]) > elvMin)\
                    and (tmp[11][:2]==bande[:2]) : # Il faut que les poursuites est 
                                      #  durées 780 s pour être retenues
                prn_a="%02i"%(int(tmp[0]))
                prn.append(prn_a)
                mjd.append(float(tmp[1])+
                    timeconv.h2j(float(tmp[2][:2]),float(tmp[2][2:4]),
                                 float(tmp[2][4:6])))
                elv.append(int(tmp[3]))
                azth.append(int(tmp[4]))
                refgps.append(int(tmp[6])/10.)
        except ValueError :
            continue
        except IndexError :
            continue
    f.close()
#
    return prn,mjd,elv,azth,refgps # Retourne 5 listes
#
def makedicrpt(prn,mjd,elv,azth,refgps) :
    """ Routine pour fabriquer un dictionaire à partir des data du cggtts lu
    
    Usage : rpt=makedicrpt(prn,mjd,elv,azth,refgps)

        INPUT  : 
            prn[]    : str,   N satellite
            mjd[]    : float, date MJD de la comparaison
            elv[]    : int,   elavation
            azth[]   : int,   azimuth
            refgps[] : float, difference entre la reference 
                       du recepteur et GPSTIME en ns
        OUTPUT :
            rpt{}    : 'key'        : str, N satellite
                       rpt['key'][] : [], Pour chaque satellite :
                                      mjd,elv,azth,refgps
    """    
    rpt={}
    nt=len(prn)
    for i in range(nt) :
        key=prn[i]
        try :
            if key in rpt.keys() :
                rpt[key].append([mjd[i],elv[i],azth[i],refgps[i]])
            else :
                rpt[key]=[[mjd[i],elv[i],azth[i],refgps[i]]]
        except IndexError :
            continue
#
    return rpt
#
def makecomprpt(rpt1,rpt2) :
    """ Routine de comparaison des vues communes
        Fabrication d'un dictionnaire avec les différences entre les recepteurs

    Usage comp=comprpt(rpt1,rpt2)
        
        INPUT  : 
            rpt1{}   : 'key'         : str, N satellite du recepteur de 
                                            référence
                       rpt['key'][]  : [], pour chaque satellite :
                                       mjd,elv,azth,refgps
            rpt2{}   : 'key'         : str, N satellite du recepteur de comparé
                       rpt['key'][]  : [], pour chaque satellite :
                                       mjd,elv,azth,refgps
        OUTPUT :
            diff{}   : 'key'         : str, N satellite commun au deux 
                                            recepteurs
                       diff['key'][] : [], pour chaque satellite :
                                       mjd,diff 
    """
#
# Récupération d'une liste (triée) avec les sattelites communs au deux fichiers
    keys=rpt1.keys()
    keys=[k for k in keys if k in rpt2.keys()]
    keys.sort()
    diff={}
    for k in keys :
        nr1=len(rpt1[k])
        nr2=len(rpt2[k])
        for i in range(nr1) :
            for j in range(nr2) :
                if (rpt1[k][i][0]==rpt2[k][j][0]) :
                    if k in diff.keys() :
                        diff[k].append([rpt1[k][i][0],
                                        rpt2[k][j][3]-rpt1[k][i][3]])
                    else :
                        diff[k]=[[rpt1[k][i][0],rpt2[k][j][3]-rpt1[k][i][3]]]
#
    return diff
#
def makealone(rpt1) :
    """ Routine de comparaison dans le cas un seul recepteur

    Usage comp=comprpt(rpt1)

        INPUT  : 
            rpt1{}   : 'key'         : str, N satellite du recepteur de 
                                            référence
                       rpt['key'][]  : [], pour chaque satellite :
                                       mjd,elv,azth,refgps

        OUTPUT :
            diff{}   : 'key'         : str, N satellite 
                       diff['key'][] : [], pour le satellite :
                                       mjd,diff 
    """
#
    keys=list(rpt1.keys())
    keys.sort()
    diff={}
    for k in keys :
        nr1=len(rpt1[k])
        for i in range(nr1) :
            if k in diff.keys() :
                diff[k].append([rpt1[k][i][0],rpt1[k][i][3]])
            else :
                diff[k]=[[rpt1[k][i][0],rpt1[k][i][3]]]
#
    return diff
#
def fit_day(diff,ordre) :
    """ Routine pour faire un fit sur le dictionnaire 
    contenant les différences rpt1-rpt2 sur une journée

    Usage coef_fit=fit_day(diff,ordre)

        INPUT  : 
            diff{}   : 'key'         : str, N satellite commun au deux 
                                            recepteurs
                       diff['key'][] : [], pour chaque satellite :
                                       mjd,diff 
            ordre    : int, ordre de l'interpolation polynomiale
        OUTPUT : 
            p[]      : float, coefficients polynomiaux, 
                       puissance la plus elevée en premier
    """
#
    couple=[]
    for k in diff.keys() :
        for d in diff[k] :
            couple.append(d)
    couple.sort()
    x=[]
    y=[]
    for c in couple :
        x.append(c[0])
        y.append(c[1])
    p=np.polyfit(x,y,ordre)
#    
    return p
#
def clean_diff(diff,p,ns) :
    """ Routine permettant le filtrage à ns*sig des différence rpt1-rpt2

    Usage diff_fil=clean_diff(diff,p,ns)

        INPUT  : 
            diff{}     : 'key'             : str, N satellite commun au deux 
                                                  recepteurs
                         diff['key'][]     : [], pour chaque satellite :
                                             mjd,diff 
            p[]        : float, coefficients polynomiaux, 
                       puissance la plus elevée en premier
            ns         : int, nombre de facteur de sigma pour le filtre
        OUTPUT :
            diff_fil{} : 'key'             : str, N satellite commun au deux 
                                                  recepteurs
                         diff_fil['key'][] : [], pour chaque satellite :
                                             mjd,diff 
            flag       : logical, True  --> 1 satellite au moins de retiré
                                  False --> 0 satellite de retiré
            nxcl       : int, nombre de poursuites retirées

    """
#
    nxcl=0
    flag=False
    sig=0.
    n=0
    nlp=len(p)
    couple=[]
    for k in diff.keys() :
        for c in diff[k] :
            couple.append([c[0],c[1],k])
    couple.sort()
    for c in couple :
        f_i=0.
        for i in range(nlp) :  # boucle sur l'ordre du fit
            f_i+=(c[0]**i)*p[nlp-1-i] # Calcul de la valeur de l'horloge 
                                      # par l'interpolation à la date mjd (c[0])
        sig+=(c[1]-f_i)**2 # sig : ecart type calculé par rapport à la 
                           # difference (ref-op)-fit
        n+=1
    sig=np.sqrt(sig/float(n))
    if(sig>2.e5) : # Si le sigma se la journée est trop élevé on fixe le filtre 
                   # à 3*sigma
        nfil=1
    else : # sinon on prend la valeur du fichier de config
        nfil=ns
    fil=nfil*sig
    diff_fil={}
    for c in couple :
        f_i=0.
        for i in range(nlp) :
            f_i+=(c[0]**i)*p[nlp-1-i]
        if np.abs(f_i-c[1])>=fil :
            flag=True
            nxcl+=1
        else :
            if c[2] in diff_fil.keys() :
                diff_fil[c[2]].append([c[0],c[1]])
            else :
                diff_fil[c[2]]=[[c[0],c[1]]]
#
    if len(diff_fil)==0 :
        print('# Toutes les poursuites sont à retirées, filtrage annulée')
        return diff,False,0
    else :
        return diff_fil,flag,nxcl
#
def mad_diff(diff,ns) :
    """  Routine permettant le filtrage à des différence rpt1-rpt2
    par une méthode MAD

    Usage : diff,ntxcl=cv.mad_diff(diff,ns)

        INPUT  : 
            diff{}     : 'key'         : str, N satellite commun au deux 
                                              recepteurs
                         diff['key'][] : [], pour chaque satellite :
                                             mjd,diff 
            ns         : int, nombre de facteur de sigma pour le filtre
        OUTPUT :
            diff{}     : 'key'         : str, N satellite commun au deux 
                                              recepteurs
                         diff['key'][] : [], pour chaque satellite :
                                             mjd,diff après supression des 
                                             outliers
            nxcl       : int, nombre de poursuites retirées

    """
#
    nxcl=0
    for k in diff.keys() :
        couple=[]
        for c in diff[k] :
            couple.append(c)
        couple.sort(key=itemgetter(1))
        N=len(couple)
        if(N%2==1) :
            xc=couple[int(N/2)][1]
        else :
            xc=0.5*(couple[int(N/2)-1][1]+couple[int(N/2)][1])
        mediane=[]
        for c in couple :
            ecart=abs(c[1]-xc)
            mediane.append(ecart)
        mediane.sort()
        if(N%2==1) :
            S=1.4826*mediane[int(N/2)]
        else :
            S=0.5*(1.4826*mediane[int(N/2)-1]+1.4826*mediane[int(N/2)])
        fil=ns*S
        for c in couple :
            if(abs(c[1]-xc)>fil) :
               diff[k].remove(c)
               nxcl+=1
#
    return diff,nxcl
#
def mean_epoque(diff) :
    """ Routine permettant de générer une moyenne sur des différences 
        rpt2-rpt1 pour chaque époque

        INPUT  : 
            diff{}     : 'key'         : str, N satellite commun au deux 
                                              recepteurs
                         diff['key'][] : [], pour chaque satellite :
                                             mjd,diff
        OUTPUT :
            epoque[]   : [mjd,<diff>,sig,np]  
                            mjd    : float, date MJD de l'époque
                            <diff> : float, moyenne sur une époque des 
                                            différences rpt2-rpt1
                            sig    : float, ecart-type calculé sur chaque époque
                            np     : nombre de poursuites par époque
    """
#
    couple=[]
    for k in diff.keys() :
        for c in diff[k] :
            couple.append(c)
    couple.sort()
    j=0
    epoque=[]
    nc=len(couple)
    mean=couple[0][1]
    j=1
    for i in range(1,nc) :
        if couple[i][0]==couple[i-1][0] : # Tant qu'on est sur la même date 
                                          # -> c'est la même époque
            mean+=couple[i][1]            # on moyenne 
            j+=1
        else :
            mean/=float(j)
            sigepo=0.
            for k in range(i-1-j,i+1) :
                sigepo+=(1./float(j))*((couple[i][1]-mean)**2)
            epoque.append([couple[i-1][0],mean,np.sqrt(sigepo),j])
            mean=couple[i][1]
            j=1
#
    return epoque
#
def filt_epoque(epoque,ordre) :
    """ Routine permettant de filtré les data moyennées par époque à 5 sigma

        Usage : epoque=filt_epoque(epoque,ordre)

        INPUT  :
            epoque[]   : [mjd,<diff>,sig,np]  
                            mjd    : float, date MJD de l'époque
                            <diff> : float, moyenne sur une époque des 
                                            différences rpt2-rpt1
                            sig    : float, ecart-type calculé sur chaque époque
                            np     : nombre de poursuites par époque
            ordre      : int, ordre du fit d'interpolation
        OUTPUT :
            epoque[]   : [mjd,<diff>,sig,np]  
                            mjd    : float, date MJD de l'époque
                            <diff> : float, moyenne sur une époque des 
                                            différences rpt2-rpt1
                            sig    : float, ecart-type calculé sur chaque époque
                            np     : nombre de poursuites par époque
            flag       : logical, True  --> 1 epoque au moins de retirée
                                  False --> 0 époque de retirée
            nxcl       : int, nombre de époque retirées
    """
#
    flag=False
    nepxcl=0
    x=[]
    y=[]
    for e in epoque :
        x.append(e[0])
        y.append(e[1])
    p=np.polyfit(x,y,ordre)
    sig=0.
    nlp=len(p)
    n=0
    for e in epoque :
        f_i=0.
        for i in range(nlp) :
            f_i+=(e[0]**i)*p[nlp-1-i]
        sig+=(f_i-e[1])**2
        n+=1
    sig=np.sqrt(sig/float(n))
    fil=3.*sig
    for e in epoque :
        f_i=0.
        for i in range(nlp) :
            f_i+=(e[0]**i)*p[nlp-1-i]
        if np.abs(f_i-e[1])>=fil :
            epoque.remove(e)
            flag=True
            nepxcl+=1
#
    return epoque,flag,nepxcl

def extra_epoques(epo,ordre,mjd) :
#
    """ Routine permettant de faire un fit sur une liste contenant les moyennes 
        par époque et d'extrapolé deux valeurs de comparaison par jour 
        14h et 00h

        Usage : mean_day=extra_epoques(epo,ordre)

    """
#
    x=[]
    y=[]
    for e in epo :
        x.append(e[0])
        y.append(e[1])
    if (len(x)>2*ordre+1) :
        p=np.polyfit(x,y,ordre)
        nlp=len(p)
        mjd00=mjd
        mjd14=mjd+((1./24.)*14.)
        phi00=0.
        phi14=0.
        for i in range(nlp) :
            phi00+=(mjd00**i)*p[nlp-1-i]
            phi14+=(mjd14**i)*p[nlp-1-i]
#
        return [mjd,phi00,phi14]
    else :
        return []
#
